from django.utils import timezone
from dateutil.relativedelta import relativedelta

from users import serializers
from users import models

from rest_framework import decorators
from rest_framework import response
from rest_framework import mixins
from rest_framework import status
from rest_framework import viewsets
from rest_framework import filters



class RecoveryTokenFilter(filters.BaseFilterBackend):
  def filter_queryset(self, request, queryset, view):
    return queryset

  def get_fields(self, view):
    return ['email']


class RecoveryTokenViewSet(viewsets.GenericViewSet):
  """
  CurrentUserViewset resource endpoint
  """
  queryset = models.User.objects.all()
  filter_backends = (RecoveryTokenFilter,)
  serializer_class = serializers.RecoveryTokenSerializer

  def create(self, request, *args, **kwargs):
    email = request.data.get('email', None)

    try:
      user = self.get_queryset().get(email=email)
    except:
      user = None

    if user:
      # Allow only 5 requests per hour
      limit = 5
      now = timezone.now()
      to_check = (now - relativedelta(hours=1)).replace(tzinfo=timezone.utc)
      tokens = models.PasswordRecoveryToken.objects.filter(user=user, created_date__gte=to_check)

      if tokens.count() >= limit:
        will_release = tokens.order_by('-created_date')[limit-1].created_date + relativedelta(hours=1)
        seconds = abs((will_release - now).seconds)
        return response.Response({'success': False, 'message': 'Five tokens generated last hour.', 'try_again_in': seconds}, status=status.HTTP_429_TOO_MANY_REQUESTS)

      token = models.PasswordRecoveryToken(user=user)
      token.save()
      user.mailing().sendRecoveryToken({'token': token.token})

    return response.Response({'success': True, 'message': 'Token requested successfully(if user exists).'})


class RecoverPasswordFilter(filters.BaseFilterBackend):
  def filter_queryset(self, request, queryset, view):
    return queryset

  def get_fields(self, view):
    return ['email']


class RecoverPasswordViewSet(viewsets.GenericViewSet):
  """
  CurrentUserViewset resource endpoint
  """
  queryset = models.PasswordRecoveryToken.objects.all()
  filter_backends = (RecoverPasswordFilter,)
  serializer_class = serializers.RecoverPasswordSerializer

  def create(self, request, *args, **kwargs):
    email = request.data.get('email', None)
    token = request.data.get('token', None)
    new_password = request.data.get('new_password', None)

    try:
      rt = self.get_queryset().get(user__email=email, token=token)
    except:
      rt = None

    if rt:
      pass
    else:
      return response.Response({'message': 'Invalid email or token.'}, status=status.HTTP_401_UNAUTHORIZED)

    return response.Response({'message': 'Password updated.'})




# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-09-13 05:37
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0003_auto_20160907_2123'),
    ]

    operations = [
        migrations.CreateModel(
            name='PasswordRecoveryToken',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('token', models.CharField(max_length=128, verbose_name='Token')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from users import serializers
from users import models

from rest_framework import decorators
from rest_framework import mixins
from rest_framework import response
from rest_framework import viewsets
from rest_framework import permissions
from rest_framework import pagination

class UserCreateViewSet(mixins.CreateModelMixin, viewsets.GenericViewSet):
  """
  UserCreateViewset resource endpoint
  """
  serializer_class = serializers.UserCreateSerializer
  queryset = models.User.objects.all()


class CurrentUserViewSet(viewsets.GenericViewSet):
  """
  CurrentUserViewset resource endpoint
  """
  serializer_class = serializers.UserSearchSerializer
  queryset = models.User.objects.all()
  permission_classes = (permissions.IsAuthenticated,)
  pagination_class = pagination.BasePagination

  def list(self, request, *args, **kwargs):
      queryset = self.get_queryset().get(pk=request.user.pk)
      serializer = self.get_serializer(queryset)
      return response.Response(serializer.data)



from django.core.mail import EmailMultiAlternatives
from django.template import Context, Template
from django.template.loader import get_template

from utils.router import ClientRouter


class BaseMail:
  """
  This class is responsible for firing emails
  """
  from_email = ''

  def __init__(self, user):
    self.user = user

  def sendEmail(self, template_name, subject, context):
    ctx = Context(context)
    text_content = get_template('email/{}.txt'.format(template_name)).render(ctx)
    html_content = get_template('email/{}.html'.format(template_name)).render(ctx)

    msg = EmailMultiAlternatives(subject, text_content, self.from_email, [self.user.email])
    msg.attach_alternative(text_content, "text/plain")
    msg.attach_alternative(html_content, "text/html")
    return msg.send() > 0

class UserMail(BaseMail):
  """
  This class is responsible for firing emails for Users
  """
  def sendRecoveryToken(self, context):
    """
    Sent when volunteer requests recovery token
    """
    c = {'link': ClientRouter.recoverPassword(context['token'])}
    return self.sendEmail('recoveryToken', 'Solicitação de recuperação de senha', c)



from django.conf.urls import url, include
from rest_framework import routers
from rest_framework_jwt.views import obtain_jwt_token
from users import views
from users import recover_password as rp

router = routers.DefaultRouter()
router.register(r'users', views.UserCreateViewSet, 'user')
router.register(r'users/current-user', views.CurrentUserViewSet, 'user-search')
router.register(r'users/recovery-token', rp.RecoveryTokenViewSet, 'user-search')
router.register(r'users/recover-password', rp.RecoverPasswordViewSet, 'user-search')

urlpatterns = [
  url(r'^', include(router.urls)),
  url(r'^api-token-auth/', obtain_jwt_token),
]



from users import models
from rest_framework import serializers

class UserCreateSerializer(serializers.ModelSerializer):
  class Meta:
    model = models.User
    fields = ['name', 'email', 'password']

class RecoveryTokenSerializer(serializers.Serializer):
  email = serializers.CharField(required=True)

  class Meta:
    fields = ['email']

class RecoverPasswordSerializer(serializers.Serializer):
  email = serializers.CharField(required=True)
  token = serializers.CharField(required=True)
  new_password = serializers.CharField(required=True)

  class Meta:
    fields = ['email', 'token', 'new_password']



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-09-07 21:23
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0002_auto_20160907_2116'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='slug',
            field=models.SlugField(blank=True, max_length=100, null=True, unique=True, verbose_name='Slug'),
        ),
    ]



import uuid

from users import emails

from django.contrib.auth.models import AbstractBaseUser
from django.contrib.auth.models import BaseUserManager

from django.db import models
from django.utils import timezone


class UserManager(BaseUserManager):
  def create_user(self, email, password=None, **extra_fields):
    now = timezone.now()
    if not email:
        raise ValueError('The given email address must be set')
    email = UserManager.normalize_email(email)
    user = self.model(email=email, is_staff=False,
                      is_active=True, last_login=now,
                      joined_date=now, **extra_fields)

    user.set_password(password)
    user.save()
    return user

  def create_superuser(self, email, password, **extra_fields):
    user = self.create_user(email, password, **extra_fields)
    user.is_staff = True
    user.is_active = True
    user.is_superuser = True
    user.save()
    return user

  class Meta:
    app_label = 'atados_core'

class User(AbstractBaseUser):
  email = models.EmailField('Email', max_length=254, unique=True)
  name = models.CharField('Name', max_length=200, null=False, blank=False)
  slug = models.SlugField('Slug', max_length=100, null=True, blank=True, unique=True)

  is_staff = models.BooleanField('Staff', default=False)
  is_active = models.BooleanField('Active', default=True)
  is_email_verified = models.BooleanField('Email verified', default=False)

  joined_date = models.DateTimeField(auto_now_add=True, null=True, blank=True)
  modified_date = models.DateTimeField(auto_now=True, null=True, blank=True)

  def mailing(self):
    return emails.UserMail(self)

  objects = UserManager()
  USERNAME_FIELD = 'email'

class PasswordRecoveryToken(models.Model):
  user = models.ForeignKey('User', blank=True, null=True)
  token = models.CharField('Token', max_length=128, null=False, blank=False)
  created_date = models.DateTimeField(auto_now_add=True, blank=True, null=True)

  def save(self, *args, **kwargs):
    if not self.pk:
      self.token = uuid.uuid4()

    super(PasswordRecoveryToken, self).save(*args, **kwargs)
